{
Client Access Objects 1.5

caoDataQueueDataAttr component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbDataQueueDataAttr object is used for setting and retrieving attributes
of data to be read from, to be written to, or already retrieved from an AS/400
data queue.  For example, you can specify whether or not data should be
converted between ASCII and EBCDIC.  Also, you can provide a key with data to
be written, or retrieve the key from data that was read.  This object is used
in conjunction with the cwbDataQueue object.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoDataQueueDataAttr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoDataQueueDataAttr exception object

  EcaoDataQueueDataAttr = class(Exception);

  TcaoSearchOrder = (
    caodqEqual,
    caodqNotEqual,
    caodqGtOrEqual,
    caodqGreater,
    caodqLtOrEqual,
    caodqLess);

  TcaoDataQueueDataAttr = class(TComponent)
  private
    { Private declarations }
    FReturnCode: integer;
    FSetupCalled: boolean;
    FsearchOrder: TcaoSearchOrder;
    Fconvert: WordBool;
    FcwbDataQueueDataAttr: cwbDataQueueDataAttr;
    function GetSearchOrder: TcaoSearchOrder;
    procedure SetSearchOrder(SrchOrd: TcaoSearchOrder);
    function GetConvert: WordBool;
    procedure SetConvert(Conv: WordBool);
  protected
    { Protected declarations }
    function ResultCodeText: string; virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Setup; virtual;
    procedure GetKey(var key: OleVariant; var keyLength: Smallint); virtual;
    procedure GetRetKey(var key: OleVariant; var keyLength: Smallint); virtual;
    procedure SetKey(key: OleVariant; keyLength: Smallint); virtual;
    procedure GetSenderInfo(var userName, userProfileName, jobName, jobNumber: WideString); virtual;
  published
    { Published declarations }
    property ReturnCode: integer read FReturnCode default cwbOK;
    property searchOrder: TcaoSearchOrder read GetSearchOrder write SetSearchOrder default caodqEqual;
    property convert: WordBool read GetConvert write SetConvert default false;
    property OleObject: cwbDataQueueDataAttr read FcwbDataQueueDataAttr;
  end;

implementation

{
Create OLE object and initilize properties.
}

constructor TcaoDataQueueDataAttr.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbDataQueueDataAttr := CocwbDataQueueDataAttr.Create;
  FSetupCalled := false;
  FsearchOrder := caodqEqual;
  Fconvert := false
end;

{
Return formatted result code text.
}

function TcaoDataQueueDataAttr.ResultCodeText: string;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
This property contains the search order to be used when reading or peeking a
keyed data queue.  The search order identifies the relationship between the key
of the record to retrieve and the key value specified on the cwbdqSetKey
method.  Possible values are cwbdqEqual, cwbdqNotEqual, cwbdqGtOrEqual,
cwbdqGreater, cwbdqLtOrEqual, or cwbdqLess.  If the data queue order is not
cwbdqSeqKeyed, this property is ignored.  If the searchOrder is not set,
cwbdqEqual is used.  This is a read/write property.

Get OLE property searchOrder if setup was called.  Otherwise, get private
variable.
}

function TcaoDataQueueDataAttr.GetSearchOrder: TcaoSearchOrder;
begin
  if FSetupCalled then
    case FcwbDataQueueDataAttr.searchOrder of
    cwbdqEqual:
      Result := caodqEqual;
    cwbdqNotEqual:
      Result := caodqNotEqual;
    cwbdqGtOrEqual:
      Result := caodqGtOrEqual;
    cwbdqGreater:
      Result := caodqGreater;
    cwbdqLtOrEqual:
      Result := caodqLtOrEqual;
    cwbdqLess:
      Result := caodqLess
    else
      Result := caodqEqual
    end
  else
    Result := FsearchOrder
end;

{
Set OLE property searchOrder if setup was called. Otherwise, set private
variable.
}

procedure TcaoDataQueueDataAttr.SetSearchOrder(SrchOrd: TcaoSearchOrder);
begin
  if FSetupCalled then
    case SrchOrd of
      caodqEqual:
        FcwbDataQueueDataAttr.searchOrder := cwbdqEqual;
      caodqNotEqual:
        FcwbDataQueueDataAttr.searchOrder := cwbdqNotEqual;
      caodqGtOrEqual:
        FcwbDataQueueDataAttr.searchOrder := cwbdqGtOrEqual;
      caodqGreater:
        FcwbDataQueueDataAttr.searchOrder := cwbdqGreater;
      caodqLtOrEqual:
        FcwbDataQueueDataAttr.searchOrder := cwbdqLtOrEqual;
      caodqLess:
        FcwbDataQueueDataAttr.searchOrder := cwbdqLess
    end
  else
    FsearchOrder := SrchOrd
end;

{
This property contains the convert flag.  If the flag is set, data being
written is converted from ASCII to EBCDIC and data being read is converted from
EBCDIC to ASCII.  If this property is not set, no conversion of the data is
performed.  This property is a read/write property.

Get OLE property convert if setup was called.  Otherwise, get private variable.
}

function TcaoDataQueueDataAttr.GetConvert: WordBool;
begin
  if FSetupCalled then
    Result :=  FcwbDataQueueDataAttr.convert
  else
    Result := Fconvert
end;

{
Set OLE property convert if setup was called. Otherwise, set private variable.
}

procedure TcaoDataQueueDataAttr.SetConvert(Conv: WordBool);
begin
  if FSetupCalled then
    FcwbDataQueueDataAttr.convert := Conv
  else
    Fconvert := Conv
end;

{
This method initializes the cwbDataQueueDataAttr object.  It must be called
before any other method or before accessing any of the objects properties.
}

procedure TcaoDataQueueDataAttr.Setup;
begin
  FReturnCode := FcwbDataQueueDataAttr.cwbdqSetup;
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueueDataAttr.Create(ResultCodeText);
  FSetupCalled := true;
  // Set OLE properties from variables
  SetSearchOrder(FsearchOrder);
  SetConvert(Fconvert)
end;

{
This method retrieves the key set by the cwbdqSetKey method.  This key is used
for writing data to a keyed data queue.  Along with the searchOrder, this key
is used to read data from a keyed data queue.  The key associated with a
retrieved record can be obtained by calling the cwbdqGetRetKey method.

Parameters:

key - Byte Array Variant

The buffer to be used to store the key value.  The KeySize property of the
cwbDataQueueAttr object can be retrieved to determine the size of the buffer
needed.  (This parameter should be passed as a SafeArray of unisgned
characters.)

keyLength - Pointer to Short Integer

On output, it contains the size of the key stored in the key buffer.
}

procedure TcaoDataQueueDataAttr.GetKey(var key: OleVariant; var keyLength: Smallint);
begin
  FReturnCode := FcwbDataQueueDataAttr.cwbdqGetKey(
  key,keyLength);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueueDataAttr.Create(ResultCodeText)
end;

{
This method retrieves the key associated with a record that was retrieved from
a keyed data queue.  If the searchOrder property has a value other than
cwbdqEqual, this key may be different than the one set by the cwbdqSetKey
method.

Parameters:

key - Byte Array Variant

The buffer to be used to store the key value.  The KeySize property of the
cwbDataQueueAttr object can be retrieved to determine the size of the buffer
needed.  (This parameter should be passed as a SafeArray of unisgned
characters.)

keyLength - Pointer to Short Integer

On output, it contains the size of the key stored in the key buffer.
}

procedure TcaoDataQueueDataAttr.GetRetKey(var key: OleVariant; var keyLength: Smallint);
begin
  FReturnCode := FcwbDataQueueDataAttr.cwbdqGetRetKey(
  key,keyLength);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueueDataAttr.Create(ResultCodeText)
end;

{
This method sets the key to be used for writing to a keyed data queue.  This
method is also used to set the key to be used, along with the searchOrder
property, for reading from a keyed data queue.

Parameters:

key - Byte Array Variant

The key value to use.  (This parameter should be passed as a SafeArray of
unsigned characters.)

keyLength - Pointer to Short Integer

The key length of the key value.
}

procedure TcaoDataQueueDataAttr.SetKey(key: OleVariant; keyLength: Smallint);
begin
  FReturnCode := FcwbDataQueueDataAttr.cwbdqSetKey(
  key,keyLength);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueueDataAttr.Create(ResultCodeText)
end;

{
This method retrieves information about the user who wrote the most recently
retrieved record to the queue.  This information is only available if the
senderID property was set to true on the cwbDataQueueAttr object used to create
the queue.

Parameters:

userName - Pointer to String

Provides a location to store the name of the user who wrote the record to the
queue.  (This should be passed as a pointer to a BSTR)

userProfileName - Pointer to String

Provides a location to store the user profile of the user who wrote the record
to the queue.  (This should be passed as a pointer to a BSTR)

jobName - Pointer to String

Provides a location to store the name of the job which wrote the record to the
queue.  (This should be passed as a pointer to a BSTR)

jobNumber - Pointer to String

Provides a location to store the ID number of the job which wrote the record to
the queue.  (This should be passed as a pointer to a BSTR)
}

procedure TcaoDataQueueDataAttr.GetSenderInfo(var userName, userProfileName, jobName, jobNumber: WideString);
begin
  FReturnCode := FcwbDataQueueDataAttr.cwbdqGetSenderInfo(
  userName,userProfileName,jobName,jobNumber);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDataQueueDataAttr.Create(ResultCodeText)
end;

end.
