{
Client Access Objects 1.5

caoDatabaseDataFormat component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbDatabaseDataFormat automation object holds the description of the data
returned as the result of a cwbDatabaseSQL or cwbDatabaseCatalog method call.
This object also allows you, in some cases, to specify how the data will be
presented, before it is requested.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoDatabaseDataFormat;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  caoErrorMessage,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoDatabaseDataFormat exception object

  EcaoDatabaseDataFormat = class(Exception);

  TcaoDfSystemLocation = (
    caodbSystem,
    caodbLocal);

  // TcaoDatabaseDataFormat component

  TcaoDatabaseDataFormat = class(TComponent)
  private
    { Private declarations }
    FReturnCode: Integer;
    FcwbDatabaseDataFormat: cwbDatabaseDataFormat;
    FcaoErrorMessage: TcaoErrorMessage;
    function GetRowSize: Integer;
    function GetLocation: TcaoDfSystemLocation;
    procedure SetLocation(Loc: TcaoDfSystemLocation);
    function GetConversionIndicator: WordBool;
    procedure SetConversionIndicator(ConvInd: WordBool);
    function GetColumnCount: Integer;
  protected
    { Protected declarations }
    function ResultCodeText: String; virtual;
    function GetErrMsg: String; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure GetColumnCCSID(columnPosition: Integer; var dataCCSID: Smallint); virtual;
    procedure GetColumnLength(columnPosition: Integer; var dataLength: Integer); virtual;
    procedure GetColumnPrecision(columnPosition: Integer; var dataPrecision: Smallint); virtual;
    procedure GetColumnScale(columnPosition: Integer; var dataScale: Smallint); virtual;
    procedure GetColumnType(columnPosition: Integer; var dataType: Smallint); virtual;
    procedure SetClientColumnToNumeric(columnPosition: Integer; columnType: Smallint; columnLength: Integer; columnPrecision, columnScale: Smallint); virtual;
    procedure SetClientColumnToString(columnPosition: Integer; columnType: Smallint; columnLength: Integer; columnCCSID: Smallint); virtual;
    procedure GetColumnName(columnPosition: Integer; var columnName: WideString); virtual;
  published
    { Published declarations }
    property ReturnCode: Integer read FReturnCode default cwbOK;
    property rowSize: Integer read GetRowSize;
    property location: TcaoDfSystemLocation read GetLocation write SetLocation default caodbLocal;
    property conversionIndicator: WordBool read GetConversionIndicator write SetConversionIndicator default true;
    property columnCount: Integer read GetColumnCount;
    property errorMessage: TcaoErrorMessage read FcaoErrorMessage write FcaoErrorMessage;
    property OleObject: cwbDatabaseDataFormat read FcwbDatabaseDataFormat;
  end;

implementation

{
Create OLE object and initilize properties.
}

constructor TcaoDatabaseDataFormat.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbDatabaseDataFormat := CocwbDatabaseDataFormat.Create
end;

{
Return formatted result code text.
}

function TcaoDatabaseDataFormat.ResultCodeText: String;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoDatabaseDataFormat.GetErrMsg: String;
var
  S: String;
begin
  S := FcaoErrorMessage.GetErrMsg;
  if S = '' then
    S := ResultCodeText;
  Result := S
end;

{
This property contains the size (in bytes) of one row of the data described by
this data format.  This is a read-only property.
}

function TcaoDatabaseDataFormat.GetRowSize: Integer;
begin
  Result := FcwbDatabaseDataFormat.rowSize
end;

{
This property indicates whether client or host information is to be retrieved.
Possible values are:  cwbdbSystem and cwbdbLocal.  If this property is not
set, cwbdbLocal is used.  This is a read/write property.
}

function TcaoDatabaseDataFormat.GetLocation: TcaoDfSystemLocation;
begin
  case FcwbDatabaseDataFormat.location of
    cwbdbSystem:
      Result := caodbSystem;
    cwbdbLocal:
      Result := caodbLocal
    else
      Result := caodbLocal
  end
end;

{
Set OLE property location.
}

procedure TcaoDatabaseDataFormat.SetLocation(Loc: TcaoDfSystemLocation);
begin
  case Loc of
    caodbSystem:
      FcwbDatabaseDataFormat.location := cwbdbSystem;
    caodbLocal:
      FcwbDatabaseDataFormat.location := cwbdbLocal
  end
end;

{
Set OLE property conversionIndicator.
}

procedure TcaoDatabaseDataFormat.SetConversionIndicator(ConvInd: WordBool);
begin
  FcwbDatabaseDataFormat.conversionIndicator := ConvInd
end;

{
This property contains the indicator that determines whether data is to be
converted between the client and host format.  Possible values are:

  cwbFalse (no conversion) and
  cwbTrue (conversion).

If this property is not set, cwbTrue is used.  This is a read/write property.
}

function TcaoDatabaseDataFormat.GetConversionIndicator: WordBool;
begin
  Result := FcwbDatabaseDataFormat.conversionIndicator
end;

{
This property contains the number of columns of data that are described by the
data format.  This is a read-only property.
}

function TcaoDatabaseDataFormat.GetColumnCount: Integer;
begin
  Result := FcwbDatabaseDataFormat.columnCount
end;

{
Handle caoErrorMessage component being removed.
}

procedure TcaoDatabaseDataFormat.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
  (AComponent = FcaoErrorMessage) then
      FcaoErrorMessage := nil
end;

{
This method returns the Coded Character Set Identifier (CCSID) for a specified
column of data.
}

procedure TcaoDatabaseDataFormat.GetColumnCCSID(columnPosition: Integer; var dataCCSID: Smallint);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbGetColumnCCSID(
  columnPosition,dataCCSID,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

{
This method returns the length (in bytes) of the data for a specified column.
}

procedure TcaoDatabaseDataFormat.GetColumnLength(columnPosition: Integer; var dataLength: Integer);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbGetColumnLength(
  columnPosition,dataLength,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

{
This method returns the precision (the total number of binary or decimal digits
of a number, excluding the sign) for a specified column of data.  If the column
type is not a numeric type, the column precision has no meaning.
}

procedure TcaoDatabaseDataFormat.GetColumnPrecision(columnPosition: Integer; var dataPrecision: Smallint);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbGetColumnPrecision(
  columnPosition,dataPrecision,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

{
This method returns the scale (the number of digits in the fractional part of a
packed or zoned decimal number) for a specified column of data.  If the column
type is something other than cwbdbPacked or cwbdbZoned, the column scale has no
meaning.
}

procedure TcaoDatabaseDataFormat.GetColumnScale(columnPosition: Integer; var dataScale: Smallint);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbGetColumnScale(
  columnPosition,dataScale,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

{
This method returns the data type for a specified column of data.

dataType possible values are:

	cwbdbPCNoConvertion
	cwbdbPCStringConversion
	cwbdbPCLong
	cwbdbPCShort
	cwbdbPCFloat
	cwbdbPCDouble
	cwbdbPCPacked
	cwbdbPCZoned
	cwbdbPCInvalidType
	cwbdbPCVarString
	cwbdbPCGraphic
	cwbdbPCVargraphic
}

procedure TcaoDatabaseDataFormat.GetColumnType(columnPosition: Integer; var dataType: Smallint);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbGetColumnType(
  columnPosition,dataType,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

{
Parameters:

columnPosition - Long

Specifies the relative position of the column within the row.  This position is
one-based.

columnType - Short Integer

Specifies the numeric type to be used.  Possible values are:

	cwbdbLong
	cwbdbShort
	cwbdbFloat
	cwbdbDouble
	cwbdbPacked
	cwbdbZoned

columnLength - Long

If the columnType is cwbdbPacked or cwbdbZoned, it specifies the number of bytes
to be used for the column length.  Otherwise, this parameter is ignored.

columnPrecision - Short Integer

If the columnType is cwbdbPacked or cwbdbZoned, it specifies the total number
digits, excluding the sign, to be used for the column.  Otherwise, this
parameter is ignored.

columnScale - Short Integer

If the columnType is cwbdbPacked or cwbdbZoned, it specifies the total number
digits in the fractional part of the number, to be used for the column.
Otherwise, this parameter is ignored.
}

procedure TcaoDatabaseDataFormat.SetClientColumnToNumeric(columnPosition: Integer; columnType: Smallint; columnLength: Integer; columnPrecision, columnScale: Smallint);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbSetClientColumnToNumeric(
  columnPosition,columnType,columnLength,columnPrecision,columnScale,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

{
This method provides a way to treat a numeric column as if it were a String
column.
}

procedure TcaoDatabaseDataFormat.SetClientColumnToString(columnPosition: Integer; columnType: Smallint; columnLength: Integer; columnCCSID: Smallint);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbSetClientColumnToString(
  columnPosition,columnType,columnLength,columnCCSID,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

{
This method returns the column name, if it exists, for a specified column of
data.

Parameters:

columnPosition - Long

Specifies the relative position of the column within the row.  This is a
one-based index.

columnName - Pointer to String

Provides a location in which to store the column name.  (This parameter should
be passed as a pointer to a BSTR.)
}

procedure TcaoDatabaseDataFormat.GetColumnName(columnPosition: Integer; var columnName: WideString);
begin
  FReturnCode := FcwbDatabaseDataFormat.cwbdbGetColumnName(
  columnPosition,columnName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseDataFormat.Create(ResultCodeText)
end;

end.
